unit Boxes;
// Version 1.04
// Added new way of preventing the Boxes from drifting away
// Version 1.03
// New Binary Flag system
// Version 1.02
// added LsBoxStartHidden
// Version 1.01
// Fixed the dissapearing box problem by ignoring the ParentWnd supplied on InitModuleEx
// cleandup some code
// Version 1.0

Interface
uses
  Windows,
  BoxFiles,
  Shortcuts,
  Modules,
  WharfModules,
  TextCuts,
  BoxAPI,
  Types;

Procedure BuildBox(FileName: String;SubBox: String);
Procedure DestroyBox(Box: PBox);

Implementation

Function WndProc(Wnd, Msg: Integer; wParam, lParam: Longint): Longint; stdcall;
Var
  Box: PBox;
  p: TPoint;
Begin
  WndProc := 0;
  Box := GetBoxFromWnd(First_Box,Wnd);
  Case Msg Of
    WM_ERASEBKGND:
      BitBlt(wParam,0,0,Box^.Width,Box^.Height,Box^.BackDc,0,0,SRCCOPY);
    WM_MOUSEMOVE:
      If ( GetFlag(Box^.Flags,Moveing) AND ( wParam = MK_lBUTTON ))Then
        Begin
          GetCursorPos(P);
          P.X := P.X-xOffs;
          P.Y := P.Y-yOffs;
          If NOT GetFlag(Box^.Flags,NotSnappy) Then
            Begin
              If (P.X<SnapRect.Left) then P.X := SnapRect.Left;
              If (P.Y<SnapRect.Top) then P.Y := SnapRect.Top;
              If ((P.X+Box^.Width)>SnapRect.Right) then P.X := SnapRect.Right-Box^.Width;
              If ((P.Y+Box^.Height)>SnapRect.Bottom) then P.Y := SnapRect.Bottom-Box^.Height;
            End;
          SetWindowPos(Wnd,0,P.X,P.Y,0,0,SWP_NOSIZE+SWP_NOZORDER);
        End Else
          SetFlag(Box^.Flags,Moveing,false);
    WM_LBUTTONDOWN:
      Begin
        p.x := LOWORD(lParam);
        p.y := HIWORD(lParam);
        If PtInRect(Box^.dragRect,p) then
        Begin
          SetFlag(Box^.Flags,Moveing,true);
          xOffs := P.X;
          yOffs := P.Y;
          SetCapture(Wnd);
        End;
      End;
    WM_LBUTTONUP:
      If GetFlag(Box^.Flags,Moveing) Then
      Begin
        SetFlag(Box^.Flags,Moveing,false);
        ReleaseCapture;
      End Else
        PostMessage(Command_HWND,BM_COMMAND,Box^.hMainWnd,Integer(Pchar(Box^.LeftClickBang)));
    WM_RBUTTONUP:
        PostMessage(Command_HWND,BM_COMMAND,Box^.hMainWnd,Integer(Pchar(Box^.RightClickBang)));
    WM_SYSCOMMAND:
      If (wParam = 61536) Then WndProc := 0
      Else
        WndProc := DefWindowProc(Wnd,msg,wParam,lParam);
    Else
      WndProc := DefWindowProc(Wnd,msg,wParam,lParam);
  End;
End;

Procedure BuildBox(FileName: String;SubBox: String);
Var
  Box: PBox;
  Wc: TWndClass;
  BackGround: HBitmap;
  p: TPoint;
  Buffer: String;
  BHeight, BWidth: Integer; //Background height and width
  TmpBoxPtr: Pointer;
  BackRgn: HRGN;
Begin
  TmpBoxPtr := OpenBox(FileName,SubBox);

  //If not found try to add the Litestep Path
  If (TmpBoxPtr = Nil) Then
  Begin
     TmpBoxPtr := OpenBox(LsPath+'\'+FileName,SubBox);
     If (TmpBoxPtr = Nil) Then
     Begin
       Messagebox(0,'Can''t load .Box File',PChar(FileName),0);
       Exit;
     End;
  End;

  //Prevent Double-Loading of Unique boxes
  If GetBoxBool(TmpBoxPtr,'LsBoxUnique')AND
               (FindWindow('LsBoxClass',
                PChar(GetBoxString(TmpBoxPtr,'LsBoxName','LsBox')))<>0)Then
    Begin
      ShowWindow(
        FindWindow('LsBoxClass',PChar(GetBoxString(TmpBoxPtr,'LsBoxName','LsBox'))),SW_Show);
      Closebox(TmpBoxPtr,false);
      Exit;
    End;

  New(Box);
  Box^.Next_Box  := First_Box;
  If (Box^.Next_Box<>nil) then
    Box^.Next_Box^.Prev_Box := Box;
  Box^.Prev_Box := nil;
  First_Box := Box;
  Box^.BoxPtr := TmpBoxPtr;

  With Box^ do
   Begin
     Title := GetBoxString(TmpBoxPtr,'LsBoxName','LsBox');

     xPos := GetBoxInt(BoxPtr,'LsBoxX',0);
     yPos := GetBoxInt(BoxPtr,'LsBoxY',0);

     If (xPos < 0) then
       xPos := GetSystemMetrics(SM_CXSCREEN) + xPos;

     If (yPos < 0) then
       yPos := GetSystemMetrics(SM_CYSCREEN) + yPos;

     If GetBoxBool(BoxPtr,'LsBoxPopupAtMousePos') then
     Begin
       GetCursorPos(p);
       xPos := p.x; yPos := p.y;
     End;

     Background := LoadLSImage(PChar(GetBoxString(BoxPtr,'LsBoxBackGround','LsBox.bmp')),nil);
     GetLSBitmapSize(Background,BWidth,BHeight);

     Height := GetBoxInt(BoxPtr,'LsBoxHeight',0);
     Width := GetBoxInt(BoxPtr,'LsBoxWidth',0);

     If (Height=0) then
       Height := BHeight;
     If (Width=0) then
       Width := BWidth;

     Buffer := GetBoxString(BoxPtr,'LsBoxDragRect','0 0 0 0');

     DragRect.Left := Str2Int(GetToken(Buffer,1));
     DragRect.Top := Str2Int(GetToken(Buffer,2));
     DragRect.Right := Str2Int(GetToken(Buffer,3));
     DragRect.Bottom := Str2Int(GetToken(Buffer,4));

     If GetBoxBool(BoxPtr,'LsBoxNotSnappy') then
       SetFlag(Box^.Flags,NotSnappy,true);

     LeftClickBang := GetBoxString(BoxPtr,'LsBoxOnLeftClick','!none');
     RightClickBang := GetBoxString(BoxPtr,'LsBoxOnRightClick','!popup');
   End;

  If NOT GetClassInfo(Inst,'LsBoxClass',wc) then
  Begin
    FillChar(wc,sizeof(wc), #0);
    wc.lpfnWndProc := @WndProc;
    wc.lpszClassName := 'LsBoxClass';
    wc.hInstance := Inst;
    RegisterClassA(wc);
  End;

  With Box^ do
  Begin
    hMainWnd := CreateWindowExA(
      WS_EX_TOOLWINDOW,'LsBoxClass',PChar(Title),WS_POPUP,
      xPos, yPos, Width, Height, 0, 0,Inst,nil);

    If GetBoxBool(BoxPtr,'LsBoxAlwaysOnTop') then
      SetWindowPos(hMainWnd, -1, 0, 0, 0, 0, SWP_NOSIZE+SWP_NOMOVE)
    Else
     Begin
       SetParent( hMainWnd,  FindWindow('DesktopBackgroundClass', nil));
       SetWindowPos(hMainWnd, 0, 0, 0, 0, 0, SWP_NOSIZE+SWP_NOMOVE);
     End;

    If NOT GetBoxBool(BoxPtr,'LsBoxStartHidden') then
      ShowWindow(Box^.hMainWnd,SW_Show);

    If GetBoxBool(BoxPtr,'LsBoxSticky') then
      SetWindowLong(hMainWnd,GWL_USERDATA,$49474541);
  End;

  BackRgn := BitmapToRegion(Background,RGB(255,0,255),$101010,0,0);
  SetWindowRgn(Box^.hMainWnd,BackRgn,false);
  //SetWindowRgnEx(Box^.hMainWnd,Box^.BackRgn);

  //Create the background device context
  Box^.BackDc := CreateCompatibleDC(GetDC(Box^.hMainWnd));
  SelectObject(Box^.BackDc, Background);
  DeleteObject(BackGround);

  InvalidateRgn(Box^.hMainWnd,0,true);

  Box^.First_Shortcut := nil;
  Box^.First_Module := nil;
  Box^.First_WharfModule := nil;

  BuildShortcuts(Box);
  PaintTextCuts(Box);
  LoadModules(Box);
  LoadWharfModules(Box);

  //Close the BoxCfgFile if not needed anymore
  If NOT (GetBoxBool(Box^.BoxPtr,'LsBoxRememberPosition')) then
     Box^.BoxPtr := CloseBox(Box^.BoxPtr,false);

End;

Procedure DestroyBox(Box: PBox);
Var
  Shortcut: PShortcut;
  WharfModule: PWharfModule;
  Module: PModule;
  Rect: TRect;
Begin
  With Box^ do
  Begin
    DestroyShortcuts(First_Shortcut);
    UnLoadWharfModules(First_WharfModule);
    UnLoadModules(First_Module);

    If ( Box^.BoxPtr<>nil ) Then
    Begin
      GetWindowRect(hMainWnd,Rect);
      SetBoxInt(BoxPtr,'LsBoxX',Rect.left MOD GetSystemMetrics(SM_CXSCREEN));
      SetBoxInt(BoxPtr,'LsBoxY',Rect.top MOD GetSystemMetrics(SM_CYSCREEN));
      CloseBox(BoxPtr,true);
    End;

    DestroyWindow(hMainWnd);
    //DeleteObject(backrgn);
    DeleteDC(BackDC);
  End;
End;

End.
